@@ -1,7 +1,6 @@
-# Authors:  David Goodger; Garth Kidd
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3268 $
-# Date: $Date: 2005-04-27 23:04:03 +0200 (Wed, 27 Apr 2005) $
+# $Id: DocutilsTestSupport.py 5033 2007-03-21 19:51:22Z wiemann $
+# Authors: David Goodger <goodger@python.org>;
+#          Garth Kidd <garth@deadlybloodyserious.com>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -17,6 +16,7 @@ Exports the following:
     - `tableparser` is 'docutils.parsers.rst.tableparser'
 
 :Classes:
+    - `StandardTestCase`
     - `CustomTestCase`
     - `CustomTestSuite`
     - `TransformTestCase`
@@ -43,26 +43,39 @@ __docformat__ = 'reStructuredText'
 import sys
 import os
 import unittest
-import docutils_difflib
+import re
 import inspect
+import traceback
 from pprint import pformat
 from types import UnicodeType, StringType
-import package_unittest
-import docutils
-import docutils.core
-from docutils import frontend, nodes, statemachine, urischemes, utils
-from docutils.transforms import universal
-from docutils.parsers import rst
-from docutils.parsers.rst import states, tableparser, roles, languages
-from docutils.readers import standalone, pep
-from docutils.statemachine import StringList, string2lines
+
+testroot = os.path.abspath(os.path.dirname(__file__) or os.curdir)
+os.chdir(testroot)
+sys.path.insert(0, os.path.normpath(os.path.join(testroot, '..')))
+sys.path.insert(0, testroot)
+sys.path.append(os.path.normpath(os.path.join(testroot, '..', 'extras')))
 
 try:
+    import docutils_difflib
+    import package_unittest
+    import docutils
+    import docutils.core
+    from docutils import frontend, nodes, statemachine, urischemes, utils
+    from docutils.transforms import universal
+    from docutils.parsers import rst
+    from docutils.parsers.rst import states, tableparser, roles, languages
+    from docutils.readers import standalone, pep
     from docutils.readers.python import moduleparser
-    from tokenize import generate_tokens
-    del generate_tokens
-except ImportError:      # moduleparser depends on modules added in Python 2.2
-    moduleparser = None
+    from docutils.statemachine import StringList, string2lines
+except ImportError:
+    # The importing module (usually __init__.py in one of the
+    # subdirectories) may catch ImportErrors in order to detect the
+    # absence of DocutilsTestSupport in sys.path.  Thus, ImportErrors
+    # resulting from problems with importing Docutils modules must
+    # caught here.
+    traceback.print_exc()
+    sys.exit(1)
+
 
 try:
     import mypdb as pdb
@@ -74,9 +87,6 @@ except:
 StringList.__repr__ = StringList.__str__
 
 
-testroot = os.path.abspath(os.path.dirname(__file__) or os.curdir)
-
-
 class DevNull:
 
     """Output sink."""
@@ -88,68 +98,15 @@ class DevNull:
         pass
 
 
-class CustomTestCase(unittest.TestCase):
-    
-    """
-    Helper class, providing extended functionality over unittest.TestCase.
+class StandardTestCase(unittest.TestCase):
 
-    This isn't specific to Docutils but of general use when dealing
-    with large amounts of text.  The methods failUnlessEqual and
-    failIfEqual have been overwritten to provide better support for
-    multi-line strings.  Furthermore, see the compare_output method
-    and the parameter list of __init__.
+    """
+    Helper class, providing the same interface as unittest.TestCase,
+    but with useful setUp and comparison methods.
     """
 
-    compare = docutils_difflib.Differ().compare
-    """Comparison method shared by all subclasses."""
-
-    def __init__(self, method_name, input, expected, id, run_in_debugger=0):
-        """
-        Initialise the CustomTestCase.
-
-        Arguments:
-
-        method_name -- name of test method to run.
-        input -- input to the parser.
-        expected -- expected output from the parser.
-        id -- unique test identifier, used by the test framework.
-        run_in_debugger -- if true, run this test under the pdb debugger.
-        """
-        self.id = id
-        self.input = input
-        self.expected = expected
-        self.run_in_debugger = run_in_debugger
-        
-        # Ring your mother.
-        unittest.TestCase.__init__(self, method_name)
-
-    def __str__(self):
-        """
-        Return string conversion. Overridden to give test id, in addition to
-        method name.
-        """
-        return '%s; %s' % (self.id, unittest.TestCase.__str__(self))
-
-    def __repr__(self):
-        return "<%s %s>" % (self.id, unittest.TestCase.__repr__(self))
-
-    def compare_output(self, input, output, expected):
-        """`input`, `output`, and `expected` should all be strings."""
-        if type(input) == UnicodeType:
-            input = input.encode('raw_unicode_escape')
-        if type(output) == UnicodeType:
-            output = output.encode('raw_unicode_escape')
-        if type(expected) == UnicodeType:
-            expected = expected.encode('raw_unicode_escape')
-        try:
-            self.assertEquals(output, expected)
-        except AssertionError:
-            print >>sys.stderr, '\n%s\ninput:' % (self,)
-            print >>sys.stderr, input
-            print >>sys.stderr, '-: expected\n+: output'
-            print >>sys.stderr, ''.join(self.compare(expected.splitlines(1),
-                                                     output.splitlines(1)))
-            raise
+    def setUp(self):
+        os.chdir(testroot)
 
     def failUnlessEqual(self, first, second, msg=None):
         """Fail if the two objects are unequal as determined by the '=='
@@ -167,9 +124,6 @@ class CustomTestCase(unittest.TestCase):
             raise self.failureException, \
                   (msg or '%s == %s' % _format_str(first, second))
 
-    def setUp(self):
-        os.chdir(testroot)
-
     # Synonyms for assertion methods
 
     assertEqual = assertEquals = failUnlessEqual
@@ -177,6 +131,88 @@ class CustomTestCase(unittest.TestCase):
     assertNotEqual = assertNotEquals = failIfEqual
 
 
+class CustomTestCase(StandardTestCase):
+    
+    """
+    Helper class, providing extended functionality over unittest.TestCase.
+
+    The methods failUnlessEqual and failIfEqual have been overwritten
+    to provide better support for multi-line strings.  Furthermore,
+    see the compare_output method and the parameter list of __init__.
+    """
+
+    compare = docutils_difflib.Differ().compare
+    """Comparison method shared by all subclasses."""
+
+    def __init__(self, method_name, input, expected, id, run_in_debugger=0,
+                 suite_settings=None):
+        """
+        Initialise the CustomTestCase.
+
+        Arguments:
+
+        method_name -- name of test method to run.
+        input -- input to the parser.
+        expected -- expected output from the parser.
+        id -- unique test identifier, used by the test framework.
+        run_in_debugger -- if true, run this test under the pdb debugger.
+        suite_settings -- settings overrides for this test suite.
+        """
+        self.id = id
+        self.input = input
+        self.expected = expected
+        self.run_in_debugger = run_in_debugger
+        self.suite_settings = suite_settings.copy() or {}
+        
+        # Ring your mother.
+        unittest.TestCase.__init__(self, method_name)
+
+    def __str__(self):
+        """
+        Return string conversion. Overridden to give test id, in addition to
+        method name.
+        """
+        return '%s; %s' % (self.id, unittest.TestCase.__str__(self))
+
+    def __repr__(self):
+        return "<%s %s>" % (self.id, unittest.TestCase.__repr__(self))
+
+    def clear_roles(self):
+        # Language-specific roles and roles added by the
+        # "default-role" and "role" directives are currently stored
+        # globally in the roles._roles dictionary.  This workaround
+        # empties that dictionary.
+        roles._roles = {}
+
+    def setUp(self):
+        StandardTestCase.setUp(self)
+        self.clear_roles()
+
+    def compare_output(self, input, output, expected):
+        """`input`, `output`, and `expected` should all be strings."""
+        if isinstance(input, UnicodeType):
+            input = input.encode('raw_unicode_escape')
+        if isinstance(output, UnicodeType):
+            output = output.encode('raw_unicode_escape')
+        if isinstance(expected, UnicodeType):
+            expected = expected.encode('raw_unicode_escape')
+        try:
+            self.assertEquals(output, expected)
+        except AssertionError, error:
+            print >>sys.stderr, '\n%s\ninput:' % (self,)
+            print >>sys.stderr, input
+            try:
+                comparison = ''.join(self.compare(expected.splitlines(1),
+                                                  output.splitlines(1)))
+                print >>sys.stderr, '-: expected\n+: output'
+                print >>sys.stderr, comparison
+            except AttributeError:      # expected or output not a string
+                # alternative output for non-strings:
+                print >>sys.stderr, 'expected: %r' % expected
+                print >>sys.stderr, 'output:   %r' % output
+            raise error
+
+
 class CustomTestSuite(unittest.TestSuite):
 
     """
@@ -192,15 +228,17 @@ class CustomTestSuite(unittest.TestSuite):
     next_test_case_id = 0
     """The next identifier to use for non-identified test cases."""
 
-    def __init__(self, tests=(), id=None):
+    def __init__(self, tests=(), id=None, suite_settings=None):
         """
         Initialize the CustomTestSuite.
 
         Arguments:
 
         id -- identifier for the suite, prepended to test cases.
+        suite_settings -- settings overrides for this test suite.
         """
         unittest.TestSuite.__init__(self, tests)
+        self.suite_settings = suite_settings or {}
         if id is None:
             mypath = os.path.abspath(
                 sys.modules[CustomTestSuite.__module__].__file__)
@@ -243,6 +281,9 @@ class CustomTestSuite(unittest.TestSuite):
             self.next_test_case_id += 1
         # test identifier will become suiteid.testid
         tcid = '%s: %s' % (self.id, id)
+        # suite_settings may be passed as a parameter;
+        # if not, set from attribute:
+        kwargs.setdefault('suite_settings', self.suite_settings)
         # generate and add test case
         tc = test_case_class(method_name, input, expected, tcid,
                              run_in_debugger=run_in_debugger, **kwargs)
@@ -287,7 +328,9 @@ class TransformTestCase(CustomTestCase):
     def test_transforms(self):
         if self.run_in_debugger:
             pdb.set_trace()
-        document = utils.new_document('test data', self.settings)
+        settings = self.settings.copy()
+        settings.__dict__.update(self.suite_settings)
+        document = utils.new_document('test data', settings)
         self.parser.parse(self.input, document)
         # Don't do a ``populate_from_components()`` because that would
         # enable the Transformer's default transforms.
@@ -304,7 +347,9 @@ class TransformTestCase(CustomTestCase):
         print '\n', self.id
         print '-' * 70
         print self.input
-        document = utils.new_document('test data', self.settings)
+        settings = self.settings.copy()
+        settings.__dict__.update(self.suite_settings)
+        document = utils.new_document('test data', settings)
         self.parser.parse(self.input, document)
         print '-' * 70
         print document.pformat()
@@ -326,11 +371,11 @@ class TransformTestSuite(CustomTestSuite):
     setUp and tearDown).
     """
 
-    def __init__(self, parser):
+    def __init__(self, parser, suite_settings=None):
         self.parser = parser
         """Parser shared by all test cases."""
 
-        CustomTestSuite.__init__(self)
+        CustomTestSuite.__init__(self, suite_settings=suite_settings)
 
     def generateTests(self, dict, dictname='totest',
                       testmethod='test_transforms'):
@@ -350,6 +395,11 @@ class TransformTestSuite(CustomTestSuite):
                 case = cases[casenum]
                 run_in_debugger = 0
                 if len(case)==3:
+                    # TODO: (maybe) change the 3rd argument to a dict, so it
+                    # can handle more cases by keyword ('disable', 'debug',
+                    # 'settings'), here and in other generateTests methods.
+                    # But there's also the method that
+                    # HtmlPublishPartsTestSuite uses <DJG>
                     if case[2]:
                         run_in_debugger = 1
                     else:
@@ -384,9 +434,9 @@ class ParserTestCase(CustomTestCase):
     def test_parser(self):
         if self.run_in_debugger:
             pdb.set_trace()
-        document = utils.new_document('test data', self.settings)
-        # Remove any additions made by "role" directives:
-        roles._roles = {}
+        settings = self.settings.copy()
+        settings.__dict__.update(self.suite_settings)
+        document = utils.new_document('test data', settings)
         self.parser.parse(self.input, document)
         output = document.pformat()
         self.compare_output(self.input, output, self.expected)
@@ -580,17 +630,6 @@ class PythonModuleParserTestSuite(CustomTestSuite):
     A collection of PythonModuleParserTestCase.
     """
 
-    notified = None
-
-    def __init__(self, *args, **kwargs):
-        if moduleparser is None:
-            if not self.notified:
-                print ('Tests of docutils.readers.python skipped; '
-                       'Python 2.2 or higher required.')
-                PythonModuleParserTestSuite.notified = 1
-            self.generateTests = self.generate_no_tests
-        CustomTestSuite.__init__(self, *args, **kwargs)
-
     def generateTests(self, dict, dictname='totest',
                       testmethod='test_parser'):
         """
@@ -624,8 +663,15 @@ class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
     Test case for publish.
     """
 
-    settings_default_overrides = {'_disable_config': 1, 'strict_visitor': 1}
-    writer_name = '' # override in subclasses
+    settings_default_overrides = {'_disable_config': 1,
+                                  'strict_visitor': 1}
+    writer_name = '' # set in subclasses or constructor
+
+    def __init__(self, *args, **kwargs):
+        if kwargs.has_key('writer_name'):
+            self.writer_name = kwargs['writer_name']
+            del kwargs['writer_name']
+        CustomTestCase.__init__(self, *args, **kwargs)
 
     def test_publish(self):
         if self.run_in_debugger:
@@ -635,34 +681,20 @@ class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
               reader_name='standalone',
               parser_name='restructuredtext',
               writer_name=self.writer_name,
-              settings_spec=self)
+              settings_spec=self,
+              settings_overrides=self.suite_settings)
         self.compare_output(self.input, output, self.expected)
 
 
-class LatexWriterPublishTestCase(WriterPublishTestCase):
-    """Test case for Latex writer."""
-    writer_name = 'latex'
-
-
-class PseudoXMLWriterPublishTestCase(WriterPublishTestCase):
-    """Test case for pseudo-XML writer."""
-    writer_name = 'pseudoxml'
-
-
 class PublishTestSuite(CustomTestSuite):
 
-    TEST_CLASSES = {
-        'latex': LatexWriterPublishTestCase,
-        'pseudoxml': PseudoXMLWriterPublishTestCase,
-    }
-
-    def __init__(self, writer_name):
+    def __init__(self, writer_name, suite_settings=None):
         """
-        `writer_name` is the name of the writer
-        to use.  It must be a key in `TEST_CLASSES`.
+        `writer_name` is the name of the writer to use.
         """
-        CustomTestSuite.__init__(self)
-        self.test_class = self.TEST_CLASSES[writer_name]
+        CustomTestSuite.__init__(self, suite_settings=suite_settings)
+        self.test_class = WriterPublishTestCase
+        self.writer_name = writer_name
 
     def generateTests(self, dict, dictname='totest'):
         for name, cases in dict.items():
@@ -678,13 +710,17 @@ class PublishTestSuite(CustomTestSuite):
                       self.test_class, 'test_publish',
                       input=case[0], expected=case[1],
                       id='%s[%r][%s]' % (dictname, name, casenum),
-                      run_in_debugger=run_in_debugger)
+                      run_in_debugger=run_in_debugger,
+                      # Passed to constructor of self.test_class:
+                      writer_name=self.writer_name)
 
 
 class HtmlPublishPartsTestSuite(CustomTestSuite):
 
     def generateTests(self, dict, dictname='totest'):
         for name, (settings_overrides, cases) in dict.items():
+            settings = self.suite_settings.copy()
+            settings.update(settings_overrides)
             for casenum in range(len(cases)):
                 case = cases[casenum]
                 run_in_debugger = 0
@@ -695,10 +731,10 @@ class HtmlPublishPartsTestSuite(CustomTestSuite):
                         continue
                 self.addTestCase(
                       HtmlWriterPublishPartsTestCase, 'test_publish',
-                      settings_overrides=settings_overrides,
                       input=case[0], expected=case[1],
                       id='%s[%r][%s]' % (dictname, name, casenum),
-                      run_in_debugger=run_in_debugger)
+                      run_in_debugger=run_in_debugger,
+                      suite_settings=settings)
 
 
 class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
@@ -709,12 +745,9 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
 
     writer_name = 'html'
 
-    def __init__(self, *args, **kwargs):
-        self.settings_overrides = kwargs['settings_overrides']
-        """Settings overrides to use for this test case."""
-
-        del kwargs['settings_overrides'] # only wanted here
-        CustomTestCase.__init__(self, *args, **kwargs)
+    settings_default_overrides = \
+        WriterPublishTestCase.settings_default_overrides.copy()
+    settings_default_overrides['stylesheet'] = ''
 
     def test_publish(self):
         if self.run_in_debugger:
@@ -725,12 +758,11 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
             parser_name='restructuredtext',
             writer_name=self.writer_name,
             settings_spec=self,
-            settings_overrides=self.settings_overrides)
+            settings_overrides=self.suite_settings)
         output = self.format_output(parts)
         # interpolate standard variables:
         expected = self.expected % {'version': docutils.__version__}
         self.compare_output(self.input, output, expected)
-
     
     standard_content_type_template = ('<meta http-equiv="Content-Type"'
                                       ' content="text/html; charset=%s" />\n')
@@ -741,8 +773,6 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
         standard_content_type_template
         + standard_generator_template % docutils.__version__)
     standard_meta_value = standard_html_meta_value % 'utf-8'
-    standard_stylesheet_value = ('<link rel="stylesheet" href="default.css" '
-                                 'type="text/css" />\n')
     standard_html_prolog = """\
 <?xml version="1.0" encoding="%s" ?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
@@ -750,14 +780,19 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
 
     def format_output(self, parts):
         """Minimize & standardize the output."""
-        # remove redundant parts:
+        # remove redundant parts & uninteresting parts:
         del parts['whole']
         assert parts['body'] == parts['fragment']
         del parts['body']
+        del parts['body_pre_docinfo']
+        del parts['body_prefix']
+        del parts['body_suffix']
+        del parts['head']
+        del parts['head_prefix']
+        del parts['encoding']
+        del parts['version']
         # remove standard portions:
         parts['meta'] = parts['meta'].replace(self.standard_meta_value, '')
-        if parts['stylesheet'] == self.standard_stylesheet_value:
-            del parts['stylesheet']
         parts['html_head'] = parts['html_head'].replace(
             self.standard_html_meta_value, '...')
         parts['html_prolog'] = parts['html_prolog'].replace(
@@ -809,12 +844,11 @@ def _format_str(*args):
 
     This is a helper function for CustomTestCase.
     """
-    import re
     return_tuple = []
     for i in args:
         r = repr(i)
-        if '\n' in i and (isinstance(i, StringType) or
-                          isinstance(i, UnicodeType)):
+        if ( (isinstance(i, StringType) or isinstance(i, UnicodeType))
+             and '\n' in i):
             stripped = ''
             if isinstance(i, UnicodeType):
                 # stripped = 'u' or 'U'
